/** This file is part of Domotz Agent.
 * Copyright (C) 2017  Domotz Ltd
 *
 * Domotz Agent is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Domotz Agent is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Domotz Agent.  If not, see <http://www.gnu.org/licenses/>.
 **/

var headersWhitelist = [
    'x-api-minor-version',
    'x-session-token',
    'authorization',
    'x-auth-key'
];

function parseProxyUrl(url) {
    return url.replace('/proxy/', '');
}

var proxyRedirect = function (resourceLocator, onException, onSuccess) {

    return function (req, res) {
        var headers = {};

        for (var indexHeader in headersWhitelist) {
            if (req.headers[headersWhitelist[indexHeader]]) headers[headersWhitelist[indexHeader]] = req.headers[headersWhitelist[indexHeader]];
        }

        var doNotAddBaseUrl = req.url.search('http') !== -1;
        // This is a filter to define if the proxy use the default baseUrl defined by agent, or use entire url from webapp

        resourceLocator.userRequest
            .sendWithPromise(parseProxyUrl(req.url), req.method, req.body ? req.body : {}, function (response, body) {
                onSuccess(res, response);
            }, doNotAddBaseUrl, {headers: headers, doNotFollow204: true}).catch(function (error) {
            onException(res, error);
        });
    };
};

module.exports.get = proxyRedirect;
module.exports.post = proxyRedirect;
module.exports.put = proxyRedirect;
module.exports.delete = proxyRedirect;
