/** This file is part of Domotz Agent.
 * Copyright (C) 2016  Domotz Ltd
 *
 * Domotz Agent is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Domotz Agent is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Domotz Agent.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Created by Andrea Azzarà <a.azzara@domotz.com> on 17/10/16.
 */

'use strict';

var MODULE = 'WEBSOCKET_SERVER';

function wsServer(ws) {

    var webSocket = ws;
    var port = null;
    var server = null;

    function onDataReceived(data, conn) {
        var message;
        try {
            message = JSON.parse(data);
        } catch(err) {
            console.alert("%s - error parsing command %s", MODULE, err);
            return;
        }

        var id = message.id || null;

        if (id === 'speed-test') {
            var size = message.payload.size;
            var chunks = message.payload.chunks;
            try {
                console.silly(MODULE + "Received " + data);
                var chunk = new Buffer(Math.floor(size / chunks));
                var mod = new Buffer(size % chunks);
            } catch (err) {
                console.error("%s - data parsing error: %s", MODULE, JSON.stringify(err));
                return;
            }

            var left = JSON.parse(JSON.stringify(size));

            var sendData = function () {
                try {
                    if (left > 0) {
                        console.info(MODULE + "left " + left);
                        if (left >= chunk.length) {
                            left -= chunk.length;
                            conn.sendBinary(chunk, sendData);
                        } else {
                            left = 0;
                            conn.sendBinary(new Buffer(mod), sendData);
                        }
                    }
                } catch (err) {
                    console.alert(MODULE + "error sending data: " + JSON.stringify(err));
                }
            };
            sendData();
        } else {
            console.info(MODULE + "unknown command id");
        }
    }

    function create() {
        server = webSocket.createServer(function (conn) {
            conn.on("text", function (data) {
                onDataReceived(data, conn);
            });

            conn.on("close", function (code, reason) {
                console.info("%s Connection closed. Exit Code %d. Reason %s", MODULE, code, reason);
            });

            conn.on("connect", function () {
                console.info(MODULE + "on connect");
            });

            conn.on("error", function () {
                console.info(MODULE + "on error");
            });

        });

        var socketErrorHandler = function (error) {
            console.alert(MODULE + "socket error: " + error);
        };

        server.listen(0, function () {
            /* listen on a random port */
            port = server.socket.address().port;
            server.socket.on('error', socketErrorHandler);
        });
    }

    function getPort() {
        return port;
    }

    function close() {
        if (!server) {
            console.info(MODULE + "unable to close connection: server not available");
            return;
        }
        server.close(function () {
            server = null;
        });
    }

    return {
        create: create,
        getPort: getPort,
        close: close
    };
}

module.exports.wsServer = wsServer;